// XInterfaceList.cpp: implementation of the XInterfaceList class.
//
//////////////////////////////////////////////////////////////////////

#include "XInterfaceList.h"
#include "XInterface.h"

#include <qsocketdevice.h>
#include <qlabel.h>

#if defined(WIN32)
#include <winsock2.h>
#include <ws2tcpip.h>
#define ENABLE_BROADCAST(socket)	{}
#define GET_INTERFACES(socket, localAddr, bytesReturned) \
	{	DWORD wsError; \
		wsError = WSAIoctl(socket, SIO_GET_INTERFACE_LIST, NULL, 0, &localAddr, \
                      sizeof(localAddr), (DWORD*)&bytesReturned, NULL, NULL); \
		if (wsError == SOCKET_ERROR) \
			qFatal("Cannot get interface list"); \
	}
#define GET_ADDRESS(sock, ifo, address)	\
	{	address = ntohl((*(SOCKADDR_IN*)ifo.iiAddress).sin_addr.s_addr);}
#define GET_NETMASK(sock, ifo, address)	\
	{	address = ntohl((*(SOCKADDR_IN*)ifo.iiNetmask).sin_addr.s_addr);}
#define GET_BROADCAST(sock, ifo, broadcast)	\
	{	broadcast = address  | (~netmask);}
#define GET_FLAGS(sock, ifo, flags)	\
	{ flags = *(ifo.iiFlags);}
#elif defined(__linux__)
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <linux/sockios.h>
#define IFF_POINTTOPOINT IFF_POINTOPOINT
#define INTERFACE_INFO struct ifreq
#define ENABLE_BROADCAST(socket)	{int one=1; setsockopt(socket, SOL_SOCKET, SO_BROADCAST, &one, sizeof(one));}
#define GET_INTERFACES(socket, localAddr, bytesReturned) \
	{ 	struct ifconf if_conf; \
		if_conf.ifc_len = sizeof(localAddr); \
		if_conf.ifc_buf = (char *) &localAddr[0]; \
		if ((ioctl(socket, SIOCGIFCONF, &if_conf)) == -1) \
			qFatal("Cannot get interface list"); \
		bytesReturned = if_conf.ifc_len; \
	}
#define GET_ADDRESS(sock, ifo, address)	\
	{ 	if ((ioctl(sock, SIOCGIFADDR, ifo)) != 0) \
			qFatal("Cannot get interface address."); \
		address = ntohl((*(sockaddr_in*) ifo.ifr_addr).sin_addr.s_addr); \
	}
#define GET_NETMASK(sock, ifo, address)	\
	{ 	if ((ioctl(sock, SIOCGIFNETMASK, ifo)) != 0) \
			qFatal("Cannot get interface address."); \
		address = ntohl((*(sockaddr_in*) ifo.ifr_addr).sin_addr.s_addr); \
	}
#define GET_BROADCAST(sock, ifo, address)	\
	{ 	if ((ioctl(sock, SIOCGIFBRDADDR, ifo)) != 0) \
			qFatal("Cannot get interface address."); \
			address = ntohl((*(sockaddr_in*) ifo.ifr_addr).sin_addr.s_addr); \
	}
#define GET_FLAGS(sock, ifo, flags)	\
	{ 	if ((ioctl(sock, SIOCGIFFLAGS, ifo)) != 0) \
			qFatal("Cannot get interface flags."); \
		flags = *ifo.ifr_ifru.ifru_flags; \
	}
#else
#error Unsupported Platform
#endif

/**************
 * PICTURES
 **************/

 #include "icons/ethernet.xpm"
 #include "icons/loopback.xpm"
 #include "icons/ppp.xpm"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

XInterfaceList::XInterfaceList()
{
	// Probe the stack and obtain the interface list

    	QSocketDevice qsd(QSocketDevice::Datagram);
    	int  bytesReturned;
	INTERFACE_INFO localAddr[10];
	int numLocalAddr;

	GET_INTERFACES(qsd.socket(), localAddr, bytesReturned)

	numLocalAddr = (bytesReturned/sizeof(INTERFACE_INFO));
	for (int i=0; i<numLocalAddr; i++)
	{
		int address, broadcast, netmask;
		bool is_broadcast, is_loopback, is_pointToPoint, is_multicast, is_up;
		int SetFlags;

		GET_ADDRESS(qsd.socket(), &localAddr[i], address)
 		GET_NETMASK(qsd.socket(), &localAddr[i], netmask)
 		GET_BROADCAST(qsd.socket(), &localAddr[i], broadcast)
		GET_FLAGS(qsd.socket(), &localAddr[i],SetFlags)

		if (SetFlags & IFF_UP)
			{is_up=true;} else {is_up=false;}
		if (SetFlags & IFF_BROADCAST)
			{is_broadcast = true;} else {is_broadcast=false;}
		if (SetFlags & IFF_MULTICAST)
			{is_multicast=true;} else {is_multicast=false;}
		if (SetFlags & IFF_LOOPBACK)
			{is_loopback=true;} else {is_loopback = false;}
		if (SetFlags & IFF_POINTTOPOINT)
			{is_pointToPoint = true;} else {is_pointToPoint=false;}

		//if (is_broadcast && !is_loopback && !is_pointToPoint)
			interfaces.push_back(new XInterface(address,netmask,broadcast,is_loopback,is_pointToPoint,is_broadcast, is_multicast));
	}

	PixmapEthernet = new QPixmap(ethernet_xpm);
	PixmapLoopback = new QPixmap(loopback_xpm);
	PixmapPPP = new QPixmap(ppp_xpm);
}

XInterfaceList::~XInterfaceList()
{
	for (std::vector<XInterface*>::iterator i = interfaces.begin(); i != interfaces.end(); ++i)
		delete *i;
	delete PixmapEthernet;
	delete PixmapPPP;
	delete PixmapLoopback;
}

QHostAddress* XInterfaceList::getBroadcastInterfaceAddress(int n)
{
	if (n<=(int)interfaces.size())
		return interfaces[n]->getAddress();
	else
		return 0;
}

QHostAddress* XInterfaceList::getBroadcastAddress(int n)
{
	if (n<=(int)interfaces.size())
		return interfaces[n]->getBroadcast();
	else
		return 0;
}

bool XInterfaceList::populateListBox()
{
	// Not implemented, we use a combobox.
  	 return false;
}

bool XInterfaceList::populateComboBox(QComboBox *cb)
{
	/* 	We use interface[] instead of an iterator to make sure we use	*
	 *	the same numbers as the retrieval functions. 				*/

	for (unsigned int i=0;i<interfaces.size(); i++)
	{
		QPixmap *curmap = PixmapEthernet;
		if (interfaces[i]->isPointToPoint()) curmap = PixmapPPP;
		if (interfaces[i]->isLoopback()) curmap = PixmapLoopback;
		cb->insertItem(*curmap, interfaces[i]->getAddress()->toString());
	}
	return true;
}
