#include "netclipwindow.h"

#include <qapplication.h>
#include <qstring.h>
#include <qclipboard.h>
#include <qhostaddress.h>

#include <qlineedit.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qsettings.h>
#include <qcheckbox.h>

#include "debug.h"
//#include "trayicon.h"

/* Enable broadcast on a specific socket. Linux (and possibly POSIX)
 * specific.
 */
#if defined(WIN32)
#define ENABLE_BROADCAST(socket)	{}
#elif defined(__linux__)
#include <sys/types.h>
#include <sys/socket.h>
#define ENABLE_BROADCAST(socket)	{int one=1; setsockopt(socket, SOL_SOCKET, SO_BROADCAST, &one, sizeof(one));}
#else
#error Unsupported Platform
#endif

netclipWindow::netclipWindow( QWidget* parent, const char* name)
	: netclipDialogBase( parent, name)
{
	requestRealClose = false;
	QPopupMenu *help = new QPopupMenu();

	from_network = false;
	
	QSettings conf;
	conf.setPath("argentini.org", "netclipboard");

	help->insertItem( "&About", this, SLOT(about()) );

	menu = new QMenuBar(this);
	menu->insertItem( "&Help", help );
    	menu->setSeparator( QMenuBar::InWindowsStyle );

	interfaceList.populateComboBox(interfaceComboBox);
	
	udp_port = conf.readNumEntry("/netclipboard/Network/udp_port", 35678l);
	portEdit->setText(QString::number(udp_port));

	int interface_number = conf.readNumEntry("/netclipboard/Network/interface_number", 0l);
	interfaceComboBox->setCurrentItem(interface_number);
	bcastaddr = interfaceList.getBroadcastAddress(interface_number);
	bcastinterface = interfaceList.getBroadcastInterfaceAddress(interface_number);

	bool start_minimized = conf.readBoolEntry("/netclipboard/Window/start_minimized", false);
	minimizedCheckBox->setChecked(start_minimized);

	if (bcastaddr == 0)
	{
		qFatal("No suitable interface was found.");
	}
	xDebug("Broadcasting from %s to %s on port %d",bcastinterface->toString().latin1(), bcastaddr->toString().latin1(),udp_port);

	// Create the Datagram socket
    qsd = new QSocketDevice(QSocketDevice::Datagram);
    if (!qsd->bind( *bcastinterface, udp_port))
    {
       	qFatal("Could not bind socket!");
    }
	
	ENABLE_BROADCAST(qsd->socket());

    // Create the notifier
    qsn = new QSocketNotifier(qsd->socket(),QSocketNotifier::Read );

	// Connect the Notifier to the incming data slot
    connect( qsn, SIGNAL( activated(int) ),
                this, SLOT( incomingData(int) ) );

    // Connect the Clipboard slot (after the socket!)
    connect( QApplication::clipboard(), SIGNAL( dataChanged() ),
                this, SLOT( clipboardDataChanged() ) );

    connect ( (const class QObject*)ApplyButton, SIGNAL(clicked()), this, SLOT(applyClicked()));
    
}

netclipWindow::~netclipWindow()
{
	// Disconnect signals and free dynamically allocated objects.

	disconnect( qsn, SIGNAL( activated(int) ),
                this, SLOT( incomingData(int) ) );
	
    disconnect( QApplication::clipboard(), SIGNAL( dataChanged() ),
                this, SLOT( clipboardDataChanged() ) );

	disconnect ( (const class QObject*)ApplyButton, SIGNAL(clicked()), this, SLOT(applyClicked()));

	delete bcastaddr;
	delete bcastinterface;
	qsd->close();


	delete qsd;
	delete qsn;
}


void netclipWindow::incomingData(int socket)
{
    QClipboard *cb = QApplication::clipboard();
    char buffer[8000];
    int nr;
    
	// Read the data
    nr = qsd->readBlock(buffer, 8000); 
	if (nr < 0) { nr = 0;}
	buffer[nr]='\0';
    
    xDebug( "There is nework data available on socket %d from %s", socket, (qsd->peerAddress()).toString().latin1() );
    //xDebug( "We are %s", (*bcastinterface).toString().latin1() );

    // Put in in the clipboard, check sender to avoid loop.

	if (!(qsd->peerAddress() == *bcastinterface))
	{
		xDebug("Received from network:%s[EOL]", buffer);
		from_network=true;
		cb->setText(buffer);
	}
	else
	{
		//xDebug("It's from me, no update");
	}	
 }

void netclipWindow::clipboardDataChanged()
{
    QClipboard *cb = QApplication::clipboard();
    QString text;
    char buffer[8000];
    int nw;

    // Copy text from the clipboard
    text = cb->text();
    if ( text )
    {
        xDebug( "The clipboard contains: %s[EOL]", text.ascii());
        strncpy(buffer, text.ascii(), 8000);

        // Now broadcast it...
	if (from_network)
	{
	    // Never broadcast that which came from 
	    // the network in the first place.
	    from_network = false;
	}
	else
	{
    	    nw = qsd->writeBlock(buffer, strlen(buffer), *(bcastaddr), udp_port);
    	    xDebug( "Written to network: %d of %d.", nw , strlen(buffer));
    	    if (nw == -1)
    	    {
        	qWarning ("Error: %d", qsd->error());
    	    }
	}
    }
}
 
void netclipWindow::applyClicked()
{
	QSettings conf;
	conf.setPath("argentini.org", "netclipboard");

	// Disconnect and close the QSD/QSN
	disconnect( qsn, SIGNAL( activated(int) ),
                this, SLOT( incomingData(int) ) );

	qsd->close();

	delete qsn;
	delete qsd;
	delete bcastaddr;
	delete bcastinterface;
	
	// Save new config
	conf.writeEntry("/netclipboard/Network/udp_port",(int)portEdit->text().toUInt());
	conf.writeEntry("/netclipboard/Network/interface_number",(int)interfaceComboBox->currentItem());
	conf.writeEntry("/netclipboard/Window/start_minimized",minimizedCheckBox->isChecked());

	// Start listening and broadcasting on a different port/address
	udp_port = portEdit->text().toUInt();

	bcastaddr = interfaceList.getBroadcastAddress(interfaceComboBox->currentItem());
	bcastinterface = interfaceList.getBroadcastInterfaceAddress(interfaceComboBox->currentItem());
	if (bcastaddr == 0)
	{
		qFatal("No suitable interface was found.");
	}

    qsd = new QSocketDevice(QSocketDevice::Datagram);
    if (!qsd->bind( *bcastinterface, udp_port))
    {
        qFatal("Could not bind socket!");
    }
	ENABLE_BROADCAST(qsd->socket());

    // Create the notifier
    qsn = new QSocketNotifier(qsd->socket(),QSocketNotifier::Read );

	// Connect the Notifier to the incming data slot
    connect( qsn, SIGNAL( activated(int) ),
                this, SLOT( incomingData(int) ) );

	xDebug("Broadcasting from %s to %s on port %d",bcastinterface->toString().latin1(), bcastaddr->toString().latin1(),udp_port);
}

void netclipWindow::about()
{
	QMessageBox::about(this, "About the Network Clipboard",
		QString::fromLatin1("The Network Clipboard\n\n")+
		"The Network Clipboard provides transparent cross-platform clipboard\nsynchronization over a local network.\n\n"+
		"(c) 2002-2004 Ranieri Argentini \n\n" +
		"This program may be redistributed under the terms and conditions of the\nGNU General Public License.\n\n" +
		"The copyright holder hereby permits linking to the non-commercial version of the QT library.");
}

